package com.example.kpprinter;

import android.app.Activity;
import android.widget.Toast;

import com.example.kpprinter.java.AndroidPrintSdk;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** KpprinterPlugin */
public class KpprinterPlugin implements MethodCallHandler , EventChannel.StreamHandler{
    private Activity mActivity;
    private Result mResult;
    private Result mSearchResult;
    private boolean canCallBack = true;

    public KpprinterPlugin(Activity activity) {
        mActivity = activity;
        AndroidPrintSdk.getInstance(mActivity).initPrint();

    }

    /** Plugin registration. */
    public static void registerWith(Registrar registrar) {
        KpprinterPlugin instance = new KpprinterPlugin(registrar.activity());
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "kpprinter");
        channel.setMethodCallHandler(instance);


        EventChannel printResult = new EventChannel(registrar.messenger(), "printer_printerSearched");
        printResult.setStreamHandler(instance);

    }

    @Override
    public void onMethodCall(MethodCall call,  Result result) {
        if (call.method.equals("printer_search"))
            this.mSearchResult = result;
        else
            this.mResult = result;
        switch (call.method) {
            case "printer_search":
                AndroidPrintSdk.getInstance(mActivity).searchPrint((List<Map<String, String>>) call.arguments);
                AndroidPrintSdk.getInstance(mActivity).setOnDeviceSearchListener(new AndroidPrintSdk.DeviceSearchListener() {
                    @Override
                    public void searchSuccess() {
                        if(canCallBack)
                            mSearchResult.success("true");

                    }

                    @Override
                    public void searchFail(){
                        if(canCallBack)
                            mSearchResult.success("false");

                    }
                });

                AndroidPrintSdk.getInstance(mActivity).setOnLocationPermissionListener(new AndroidPrintSdk.LocationPermissionListener() {
                    @Override
                    public void close() {
                        canCallBack = false;
                        mSearchResult.success("notGps");
                    }
                    @Override
                    public void open() {
                        canCallBack = true;
                    }
                });

                break;
            case "printer_connectAndPrint":
                AndroidPrintSdk.getInstance(mActivity).connectAndPrint((Map<String, Object>) call.arguments);



                AndroidPrintSdk.getInstance(mActivity).setOnDeviceConnectListener(new AndroidPrintSdk.DeviceConnectListener() {
                    @Override
                    public void connectFalse() {
                        mActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                mResult.success("false");
                            }
                        });

                    }

                    @Override
                    public void connectSuccess() {
                    }

                    @Override
                    public void connectUnConnect() {
                    }
                });

                AndroidPrintSdk.getInstance(mActivity).setOnPrintResultListener(new AndroidPrintSdk.PrintResultListener() {
                    @Override
                    public void printSuccess() {
                        mActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                mResult.success("true");
                            }
                        });
//
                    }

                    @Override
                    public void printFail(final String failMsg) {
                        mActivity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                mResult.success(failMsg);
                            }
                        });
                    }
                });

                break;
            case "printer_disconnectAPrinter":
                AndroidPrintSdk.getInstance(mActivity).finishView();
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onListen(Object o, final EventChannel.EventSink eventSink) {
        AndroidPrintSdk.getInstance(mActivity).setPrintDeviceMsgListener(new AndroidPrintSdk.PrintDeviceMsgListener() {
            @Override
            public void printerMsg(LinkedHashMap<String, String> map) {
                if (map.size() > 0)
                    eventSink.success(map);
                else
                    eventSink.error("", "", null);
            }
        });
    }

    @Override
    public void onCancel(Object o) {

    }
}
